import { Button, Col, Drawer, Input, message, Row, Space } from "antd";
import React, { useState, useImperativeHandle, forwardRef, useRef, useEffect, useMemo } from "react";
import PageList from "./pageList";
import Content from "./content";
import SetData from "./set";
import { useToken } from "@ant-design/pro-components";
import { useSize } from "ahooks";
import { useModel } from "@umijs/max";
import { useAjax } from "@/Hook/useAjax";
import { appComponentConfigAddOrUpdate, appComponentConfigGetAppPageList } from "@/services/miniApp/compConfig";
// 定义组件的 Props 类型
interface Props {
    listApi: any
}

// 使用 forwardRef 以支持传递 ref
const ModalForm = forwardRef((props: Props, ref) => {
    let { initialState } = useModel("@@initialState")
    let { state, dispatch } = useModel('appPageConifg')
    const [open, setOpen] = useState(false);
    const headerSize = useSize(document.querySelector('.ant-drawer-header'));
    const { token } = useToken()
    const PageListRef = useRef<any>(null);
    const ContentRef = useRef<any>(null);
    const SetDataRef = useRef<any>(null);
    const [templateName, setTemplateName] = useState("")
    const AppComponentConfigGetAppPageList = useAjax((params) => appComponentConfigGetAppPageList(params))
    const AppComponentConfigAddOrUpdate = useAjax((params) => appComponentConfigAddOrUpdate(params))//添加
    // console.log("props", props)
    // 接口公共参数
    let publicData = useMemo(() => {
        return {
            appId: initialState?.selectApp?.id || "",
            appType: initialState?.selectApp?.appType || ""
        }
    }, [initialState?.selectApp])
    // 获取当前小程序可配置的页面和组件
    useEffect(() => {
        if(open){
            AppComponentConfigGetAppPageList.run(publicData).then(res => {
                let page = res?.data?.find((page: { enabledComponent: any; }) => page.enabledComponent)
                page && dispatch({ type: "setAll", params: { activePage: page?.pagePath, isWorkDirection: true } })
            })
        }
    }, [publicData,open])
    // 编辑时替换模板名称
    useEffect(()=>{
        setTemplateName(state.templateName)
    },[state.templateName])
    // close 方法，用于关闭 Drawer
    const close = () => {
        dispatch({
            type: 'setAll', params: {
                tabs: 0,
                isWorkDirection: false,
                compAc: "",
                index: 0,
                activePage: "",
                pageConfigList: []
            }
        })
        setOpen(false); // 将 open 设置为 false，关闭 Drawer
    };

    const openDrawer = () => {
        setOpen(true); // 将 open 设置为 true，打开 Drawer
    };
    // 保存
    const save = () => {
        if (!templateName) {
            return message.error("请填写模板名称！")
        }
        AppComponentConfigAddOrUpdate.run({ templateName, pageConfigList: state.pageConfigList, ...publicData }).then(res => {
            if (res.code === 200) {
                props?.listApi?.refresh()
                AppComponentConfigGetAppPageList.refresh()
                close()
            }
        })

    }
    // 使用 useImperativeHandle 暴露方法给父组件
    useImperativeHandle(ref, () => ({
        close,        // 暴露 close 方法
        openDrawer,   // 也可以暴露打开的方法
        save
    }));
    return (
        <Drawer
            title="新建配置"
            placement="right"
            open={open}
            onClose={close}
            width={'85vw'}
            extra={<Space>
                <Input placeholder="请输入模板名称" value={templateName} allowClear onChange={(e) => {
                    let value = e.target.value
                    setTemplateName(value)
                }} />
                <Button type='primary' onClick={save}>保存配置</Button>
            </Space>}
            maskClosable={false}
        >
            <Row wrap style={{ height: `calc(100vh - ${(headerSize?.height || 0) + 50}px)` }}>
                {/* 左侧页面列表 pageList*/}
                <Col xl={5} xs={24}>
                    <PageList ref={PageListRef} data={AppComponentConfigGetAppPageList?.data?.data} />
                </Col>
                {/* 中间展示 content*/}
                <Col xl={14} xs={24} style={{ background: token.colorBgContainerDisabled, display: 'flex', alignItems: 'center', justifyContent: 'center' }}>
                    <Content ref={ContentRef} />
                </Col>
                {/* 右侧参数设置 set*/}
                <Col xl={5} xs={24}>
                    <SetData ref={SetDataRef} pageList={AppComponentConfigGetAppPageList?.data?.data} />
                </Col>
            </Row>
        </Drawer>
    );
});

export default ModalForm;